package sos.scheduler.file;


import sos.spooler.Job_impl;
import sos.spooler.Variable_set;
import sos.util.SOSLogger;
import sos.util.SOSSchedulerLogger;
import sos.util.SOSFileOperations;

/**
 * This job checks for file non-existence of one file or several files of a directory.
 * It can be used standalone or as an order driven job.
 * 
 * @author Florian Schreiber <fs@sos-berlin.com>
 * @since  2006-12-08
*/
public class JobSchedulerNotExistsFile extends Job_impl {
	
	private SOSLogger logger = null;
	
	
	/**
	 * Implementierung fr Spooler API.
	 * Initialisierungmethode fr den Scheduler.
	 * @return boolean
	 */
	public boolean spooler_init() {
		
		try {
			
			try {
				this.logger = new SOSSchedulerLogger(this.spooler_log);
			} catch (Exception e) {
				throw new Exception("error occurred instantiating logger: " + e.getMessage());
			}
			
			return true;
		} catch (Exception e) {
			try {
				if ( logger!=null ) logger.error("error occurred in spooler_init(): " + e.getMessage());
			} catch (Exception x) {}
			
			return false;
		}
	}
	
	/**
	 * Implementierung fr Spooler API. Luft bis return = false
	 * @return boolean
	 */
	public boolean spooler_process() {
		
		String name = null;
		
		String file = null;
		String fileSpec = null;
		
		try {
			
			// Job oder Order
			Variable_set params = spooler.create_variable_set();
			if (spooler_task.params() != null) params.merge(spooler_task.params());
			if (spooler_job.order_queue() != null && spooler_task.order().params() != null)
				params.merge(spooler_task.order().params());
			
			// mandatory parameters
			name = "file";
			if ( params.var(name)!=null && params.var(name).length()>0 )				
				file = params.var(name);
			else
				throw new Exception("job parameter is missing: ["+name+"]");
			
			logger.info(".. job parameter ["+name+"]: " + file);
			
			// optional parameters
			name = "file_spec";
			if ( params.var(name)!=null && params.var(name).length()>0 ) {				
				fileSpec = params.var(name);
				logger.info(".. job parameter ["+name+"]: " + fileSpec);
			}
			
			if ( SOSFileOperations.existsFile(file, fileSpec, logger) )
			    throw new Exception("check for absence failed for file: " + file);
			
			return ( spooler_job.order_queue()!=null );
			
		} catch (Exception e) {
			try {
				logger.error("error occurred in JobSchedulerNotExistsFile: " + e.getMessage());
			} catch(Exception x) {}
			
			return false;
		}
	}
	
	
	/**
	 * Implementierung fr Spooler API. Wird beim Entladen aufgerufen und
	 * schliet die Datenbankverbindung.
	 */
	public void spooler_exit() {
		
		try {
			
		} catch (Exception e) {
			// no error processing at job level
		} finally {
		}
	}
}
