package sos.scheduler.file;


import sos.spooler.Job_impl;
import sos.spooler.Variable_set;
import sos.util.SOSLogger;
import sos.util.SOSSchedulerLogger;
import sos.util.SOSFileOperations;

/**
 * This job removes one file or several files from the file system.
 * It can be used standalone or as an order driven job.
 * 
 * @author Florian Schreiber <fs@sos-berlin.com>
 * @since  2006-11-01
*/
public class JobSchedulerRemoveFile extends Job_impl {
	
	private SOSLogger logger = null;
	
	
	/**
	 * Implementierung fr Spooler API.
	 * Initialisierungmethode fr den Scheduler.
	 * @return boolean
	 */
	public boolean spooler_init() {
		
		try {
			
			try {
				this.logger = new SOSSchedulerLogger(this.spooler_log);
			} catch (Exception e) {
				throw new Exception("error occurred instantiating logger: " + e.getMessage());
			}
			
			return true;
		} catch (Exception e) {
			try {
				if ( logger!=null ) logger.error("error occurred in spooler_init(): " + e.getMessage());
			} catch (Exception x) {}
			
			return false;
		}
	}
	
	/**
	 * Implementierung fr Spooler API. Luft bis return = false
	 * @return boolean
	 */
	public boolean spooler_process() {
		
		boolean rc = false;
		String name = null;
		String file = null;
		String fileSpec = ".*";
		int flags = 0;
		
		try {
			
			// Job oder Order
			Variable_set params = spooler.create_variable_set();
			if (spooler_task.params() != null) params.merge(spooler_task.params());
			if (spooler_job.order_queue() != null && spooler_task.order().params() != null)
				params.merge(spooler_task.order().params());
			
			// mandatory parameters
			name = "file";
			if ( params.value("file") != null && params.value("file").length()>0 ) {				
				file = params.value("file");
                logger.info(".. job parameter [file]: " + file);
            } else if ( params.value("source_file") != null && params.value("source_file").length() > 0 ) {                
                file = params.value("source_file");
                logger.info(".. job parameter [source_file]: " + file);
            } else {
				throw new Exception("job parameter is missing: ["+name+"]");
            }
			
			
			// optional parameters
			name = "file_spec";
			if ( params.value(name)!=null && params.value(name).length()>0 ) {				
				fileSpec = params.value(name);
				logger.info(".. job parameter ["+name+"]: " + fileSpec);
			}
			
			try {
				
				name = "gracious";
        if ( params.value(name) != null && params.value(name).length() > 0 ) { 
					flags |= ( SOSFileOperations.toBoolean(params.value(name)) ) ? SOSFileOperations.GRACIOUS : 0;
					logger.info(".. job parameter ["+name+"]: " + params.value(name));
				}
				
        name = "recursive";
				if ( params.value(name) != null && params.value(name).length() > 0 ) { 
					flags |= ( SOSFileOperations.toBoolean(params.value(name)) ) ? SOSFileOperations.RECURSIVE : 0;
					logger.info(".. job parameter ["+name+"]: " + params.value(name));
				}
				
				name = "remove_dir";
				if ( params.value(name) != null && params.value(name).length() > 0 ) { 
					flags |= ( SOSFileOperations.toBoolean(params.value(name)) ) ? SOSFileOperations.REMOVE_DIR : 0;
					logger.info(".. job parameter ["+name+"]: " + params.value(name));
				}
				
				name = "wipe";
				if ( params.var(name)!=null && params.var(name).length()>0 ) {
					flags |= ( SOSFileOperations.toBoolean(params.var(name)) ) ? SOSFileOperations.WIPE : 0;
					logger.info(".. job parameter ["+name+"]: " + params.var(name));
				}
				
			} catch (Exception x) {
				throw new Exception("cannot evaluate job parameter ["+name+"]: "+ x.getMessage());
			}
			
			rc = SOSFileOperations.removeFile(file, fileSpec, flags, logger);
			
			if (!rc && params.value("gracious") != null && params.value("gracious").equalsIgnoreCase("all")) {
				return (spooler_job.order_queue() != null);
			} else {
				return (spooler_job.order_queue() != null) ? rc : false;
			}
			
		} catch (Exception e) {
			try {
				logger.error("error occurred in JobSchedulerRemoveFile: " + e.getMessage());
			} catch(Exception x) {}
			
			return false;
		}
	}
	
	
	/**
	 * Implementierung fr Spooler API. Wird beim Entladen aufgerufen und
	 * schliet die Datenbankverbindung.
	 */
	public void spooler_exit() {
		
		try {
			
		} catch (Exception e) {
			// no error processing at job level
		} finally {
		}
	}
}
