package sos.scheduler.ftp;


import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import sos.net.SOSFTP;
import sos.net.SOSMail;
import sos.scheduler.job.JobSchedulerJob;
import sos.spooler.Variable_set;
import sos.util.SOSFileOperations;
import sos.util.SOSSchedulerLogger;
import sos.util.SOSString;

/**
 * FTP File Transfer
 *
 * @author Andreas Pschel 
 */

public class JobSchedulerFTPReceive extends JobSchedulerJob {
	
	/** The FTP server will always reply the ftp error codes, 
	 * see http://www.the-eggman.com/seminars/ftp_error_codes.html */
	public static final int ERROR_CODE = 300; 
	
	
	public boolean spooler_process() {
		
		boolean rc = false;
		SOSFTP ftpClient = null;		
		
		SOSString sosString = new SOSString();
		
		String host = "";
		int port = 21;
		
		String user = "anonymous";
		String password = "";
		String account = "";
		
		String transferMode = "binary";
		boolean passiveMode = false;
		
		String remoteDir = "./";
		String localDir = ".";
		String fileSpec = ".*";
		String atomicSuffix = "";
		boolean checkSize = true;
		long checkInterval = 60;
		long checkRetry = 0;
		boolean overwriteFiles = true;
		boolean appendFiles = false;
		boolean removeFiles = false;
		boolean forceFiles = true;
		
		boolean zeroByteFiles = true;
		boolean zeroByteFilesStrict = false; 
		boolean zeroByteFilesRelaxed = false;
		
		String fileNotificationTo = "";
		String fileNotificationCC = "";
		String fileNotificationBCC = "";
		String fileNotificationSubject = "";
		String fileNotificationBody = "";
		
		String fileZeroByteNotificationTo = "";
		String fileZeroByteNotificationCC = "";
		String fileZeroByteNotificationBCC = "";
		String fileZeroByteNotificationSubject = "";
		String fileZeroByteNotificationBody = "";
		
		boolean isLoggedIn = false;
		Vector transferFileList = new Vector();
		HashMap checkFileList = new HashMap();
		Variable_set params = null;
		
		String replacing = null;
		String replacement = null;	
		
		boolean recursive = false;
		
		int pollTimeout=0;
		int pollIntervall=60;
		int pollMinFiles=1;
		
		String filePath = "";
		boolean isFilePath = false;
		
		try {   
			this.setLogger(new SOSSchedulerLogger(spooler_log));
			
			try { // to get the job parameters and order parameters
				params = spooler.create_variable_set();
				if (spooler_task.params() != null) params.merge(spooler_task.params());
				if (spooler_job.order_queue() != null && spooler_task.order().params() != null) params.merge(spooler_task.order().params());
				
				if (params.var("ftp_host") != null && params.var("ftp_host").length() > 0)
					host = params.var("ftp_host");
				
				if (params.var("ftp_port") != null && params.var("ftp_port").length() > 0) {
					try {
						port = Integer.parseInt(params.var("ftp_port"));
					} catch (Exception e) {
						throw new Exception("illegal value for parameter [ftp_port]: " + params.var("ftp_port"));
					}
				}
				
				if (params.var("ftp_user") != null && params.var("ftp_user").length() > 0)
					user = params.var("ftp_user");
				if (params.var("ftp_password") != null && params.var("ftp_password").length() > 0)
					password = params.var("ftp_password");
				if (params.var("ftp_account") != null && params.var("ftp_account").length() > 0)
					account = params.var("ftp_account");
				
				if (params.var("ftp_transfer_mode") != null && params.var("ftp_transfer_mode").length() > 0)
					transferMode = params.var("ftp_transfer_mode");
				
				if (params.var("ftp_passive_mode") != null && params.var("ftp_passive_mode").length() > 0)
					passiveMode = (params.var("ftp_passive_mode").equals("1") ||
							params.var("ftp_passive_mode").equalsIgnoreCase("true") ||
							params.var("ftp_passive_mode").equalsIgnoreCase("yes")? true : false);
				
				if (params.var("ftp_remote_dir") != null && params.var("ftp_remote_dir").length() > 0)
					remoteDir = params.var("ftp_remote_dir");
				
				if (params.var("ftp_local_dir") != null && params.var("ftp_local_dir").length() > 0)
					localDir = params.var("ftp_local_dir");
				
				if (params.var("ftp_file_spec") != null && params.var("ftp_file_spec").length() > 0)
					fileSpec = params.var("ftp_file_spec");
				
				if (params.var("ftp_atomic_suffix") != null && params.var("ftp_atomic_suffix").length() > 0)
					atomicSuffix = params.var("ftp_atomic_suffix");
				
				
				if (params.var("ftp_check_size") != null && params.var("ftp_check_size").length() > 0)
					checkSize = (!params.var("ftp_check_size").equals("1") 
							&& !params.var("ftp_check_size").equalsIgnoreCase("true") 
							&& !params.var("ftp_check_size").equalsIgnoreCase("yes") ? false : true);
				
				if (params.var("ftp_check_interval") != null && params.var("ftp_check_interval").length() > 0) {
					try {
						checkInterval = Long.parseLong(params.var("ftp_check_interval"));
					} catch (Exception e) {
						throw new Exception("illegal value for parameter [ftp_check_interval]: " + params.var("ftp_check_interval"));
					}
				}
				
				if (params.var("ftp_check_retry") != null && params.var("ftp_check_retry").length() > 0) {
					try {
						checkRetry = Long.parseLong(params.var("ftp_check_retry"));
					} catch (Exception e) {
						throw new Exception("illegal value for parameter [ftp_check_retry]: " + params.var("ftp_check_retry"));
					}
				}
				
				if (params.var("ftp_overwrite_files") != null && params.var("ftp_overwrite_files").length() > 0)
					overwriteFiles = (!params.var("ftp_overwrite_files").equals("1") 
							&& !params.var("ftp_overwrite_files").equalsIgnoreCase("true") 
							&& !params.var("ftp_overwrite_files").equalsIgnoreCase("yes") ? false : true);
				
				if (params.var("ftp_append_files") != null && params.var("ftp_append_files").length() > 0)
					appendFiles = (params.var("ftp_append_files").equals("1") 
							|| params.var("ftp_append_files").equalsIgnoreCase("true") 
							|| params.var("ftp_append_files").equalsIgnoreCase("yes") ? true : false);
				
				if (params.var("ftp_remove_files") != null && params.var("ftp_remove_files").length() > 0)
					removeFiles = (params.var("ftp_remove_files").equals("1") 
							|| params.var("ftp_remove_files").equalsIgnoreCase("true") 
							|| params.var("ftp_remove_files").equalsIgnoreCase("yes") ? true : false);
				
				if (params.var("ftp_force_files") != null && params.var("ftp_force_files").length() > 0)
					forceFiles = (!params.var("ftp_force_files").equals("1") 
							&& !params.var("ftp_force_files").equalsIgnoreCase("true") 
							&& !params.var("ftp_force_files").equalsIgnoreCase("yes") ? false : true);
				
				
				if (params.var("ftp_file_zero_byte_transfer") != null && params.var("ftp_file_zero_byte_transfer").length() > 0) {
					if (params.var("ftp_file_zero_byte_transfer").equals("1") 
							|| params.var("ftp_file_zero_byte_transfer").equalsIgnoreCase("true") 
							|| params.var("ftp_file_zero_byte_transfer").equalsIgnoreCase("yes")) {
						zeroByteFiles = true;
						zeroByteFilesStrict = false;
					} else if (params.var("ftp_file_zero_byte_transfer").equalsIgnoreCase("strict")) {
						zeroByteFiles = false;
						zeroByteFilesStrict = true;
					} else if (params.var("ftp_file_zero_byte_transfer").equalsIgnoreCase("relaxed")) {
						zeroByteFiles = false;
						zeroByteFilesStrict = false;
						zeroByteFilesRelaxed = true;
					} else {
						zeroByteFiles = false;
						zeroByteFilesStrict = false;
					}
				}
				
				if (params.var("ftp_file_notification_to") != null && params.var("ftp_file_notification_to").length() > 0) {
					fileNotificationTo = params.var("ftp_file_notification_to");
				}
				
				if (params.var("ftp_file_notification_cc") != null && params.var("ftp_file_notification_cc").length() > 0) {
					fileNotificationCC = params.var("ftp_file_notification_cc");
				}
				
				if (params.var("ftp_file_notification_bcc") != null && params.var("ftp_file_notification_bcc").length() > 0) {
					fileNotificationBCC = params.var("ftp_file_notification_bcc");
				}
				
				if (params.var("ftp_file_notification_subject") != null && params.var("ftp_file_notification_subject").length() > 0) {
					fileNotificationSubject = params.var("ftp_file_notification_subject");
				}
				
				if (params.var("ftp_file_notification_body") != null && params.var("ftp_file_notification_body").length() > 0) {
					fileNotificationBody = params.var("ftp_file_notification_body");
				}
				
				
				if (params.var("ftp_file_zero_byte_notification_to") != null && params.var("ftp_file_zero_byte_notification_to").length() > 0) {
					fileZeroByteNotificationTo = params.var("ftp_file_zero_byte_notification_to");
				}
				
				if (params.var("ftp_file_zero_byte_notification_cc") != null && params.var("ftp_file_zero_byte_notification_cc").length() > 0) {
					fileZeroByteNotificationCC = params.var("ftp_file_zero_byte_notification_cc");
				}
				
				if (params.var("ftp_file_zero_byte_notification_bcc") != null && params.var("ftp_file_zero_byte_notification_bcc").length() > 0) {
					fileZeroByteNotificationBCC = params.var("ftp_file_zero_byte_notification_bcc");
				}
				
				if (params.var("ftp_file_zero_byte_notification_subject") != null && params.var("ftp_file_zero_byte_notification_subject").length() > 0) {
					fileZeroByteNotificationSubject = params.var("ftp_file_zero_byte_notification_subject");
				}
				
				if (params.var("ftp_file_zero_byte_notification_body") != null && params.var("ftp_file_zero_byte_notification_body").length() > 0) {
					fileZeroByteNotificationBody = params.var("ftp_file_zero_byte_notification_body");
				}
				
				if ( params.value("replacing")!=null && params.value("replacing").length()>0 ) {				
					replacing = params.value("replacing");    				
				}
				
				if ( params.value("replacement")!=null && params.value("replacement").length()>0 ) {				
					replacement = params.value("replacement");    				
				}    
				
				if ( params.value("recursive")!=null && params.value("recursive").length()>0 ) {	
					String sRecursive = "";
					sRecursive = params.value("recursive");
					recursive = sosString.parseToBoolean(sRecursive);
				}  
				
				if ( replacing != null && replacement == null ) {
					throw new Exception("job parameter is missing for specified parameter [replacing]: [replacement]");
				}
				
				if ( replacing == null && replacement != null ) {
					throw new Exception("job parameter is missing for specified parameter [replacement]: [replacing]");
				}
				
				if ( params.value("ftp_poll_timeout")!=null && params.value("ftp_poll_timeout").length()>0 ) {				
					pollTimeout = Integer.parseInt(params.value("ftp_poll_timeout"));    				
				}
				
				if ( params.value("ftp_poll_interval")!=null && params.value("ftp_poll_interval").length()>0 ) {				
					pollIntervall = Integer.parseInt(params.value("ftp_poll_interval"));    				
				}
				
				if ( params.value("ftp_poll_minfiles")!=null && params.value("ftp_poll_minfiles").length()>0 ) {				
					pollMinFiles = Integer.parseInt(params.value("ftp_poll_minfiles"));    				
				}
				
			} catch (Exception e) {
				rc = false;
				throw (new Exception("could not process job parameters: " + e.getMessage()));
			}
			
			
			if (fileNotificationTo != null && fileNotificationTo.length() > 0) {
				if (fileNotificationSubject == null || fileNotificationSubject.length() == 0) {
					if (spooler_job.order_queue() != null) {
						fileNotificationSubject = "[info] Job Chain: " + spooler_task.order().job_chain().name() + ", Order: " + spooler_task.order().id() + ", Job: " + spooler_job.name() + " (" + spooler_job.title() + "), Task: " + spooler_task.id();
					} else {
						fileNotificationSubject = "[info] Job: " + spooler_job.name() + " (" + spooler_job.title() + "), Task: " + spooler_task.id();
					}
				}
				
				if (fileNotificationBody == null || fileNotificationBody.length() == 0) {
					fileNotificationBody = "The following files have been received:\n\n";
				}
			}
			
			if (fileZeroByteNotificationTo != null && fileZeroByteNotificationTo.length() > 0) {
				if (fileZeroByteNotificationSubject == null || fileZeroByteNotificationSubject.length() == 0) {
					if (spooler_job.order_queue() != null) {
						fileZeroByteNotificationSubject = "[warning] Job Chain: " + spooler_task.order().job_chain().name() + ", Order: " + spooler_task.order().id() + ", Job: " + spooler_job.name() + " (" + spooler_job.title() + "), Task: " + spooler_task.id();
					} else {
						fileZeroByteNotificationSubject = "[warning] Job: " + spooler_job.name() + " (" + spooler_job.title() + "), Task: " + spooler_task.id();
					}
				}
				
				if (fileZeroByteNotificationBody == null || fileZeroByteNotificationBody.length() == 0) {
					fileZeroByteNotificationBody = "The following files have been received and were removed due to zero byte constraints:\n\n";
				}
			}			
			
			if (params.var("ftp_file_path") != null && params.var("ftp_file_path").length() > 0) {
            	filePath = params.var("ftp_file_path");
            	isFilePath = true;
			} else {
				isFilePath = false;
			}
			
			this.getLogger().debug(".. job parameter [ftp_host]                : " + host);
			this.getLogger().debug(".. job parameter [ftp_port]                : " + port);
			this.getLogger().debug(".. job parameter [ftp_user]                : " + user);			
			this.getLogger().debug(".. job parameter [ftp_account]             : " + account);
			this.getLogger().debug(".. job parameter [ftp_transfer_mode]       : " + transferMode);
			this.getLogger().debug(".. job parameter [ftp_passive_mode]        : " + passiveMode);
			this.getLogger().debug(".. job parameter [ftp_remote_dir]          : " + remoteDir);
			this.getLogger().debug(".. job parameter [ftp_local_dir]           : " + localDir);
			this.getLogger().debug(".. job parameter [ftp_file_spec]           : " + fileSpec);
			this.getLogger().debug(".. job parameter [ftp_atomic_suffix]       : " + atomicSuffix);
			this.getLogger().debug(".. job parameter [ftp_check_size]          : " + checkSize);
			this.getLogger().debug(".. job parameter [ftp_check_interval]      : " + checkInterval);
			this.getLogger().debug(".. job parameter [ftp_check_retry]         : " + checkRetry);
			this.getLogger().debug(".. job parameter [ftp_overwrite_files]     : " + overwriteFiles);
			this.getLogger().debug(".. job parameter [ftp_append_files]        : " + appendFiles);
			this.getLogger().debug(".. job parameter [ftp_remove_files]        : " + removeFiles);
			this.getLogger().debug(".. job parameter [ftp_force_files]         : " + forceFiles);
			
			this.getLogger().debug(".. job parameter [ftp_zero_byte_transfer] zeroByte        : " + zeroByteFiles);
			this.getLogger().debug(".. job parameter [ftp_zero_byte_transfer] strict          : " + zeroByteFilesStrict);
			this.getLogger().debug(".. job parameter [ftp_zero_byte_transfer] relaxed         : " + zeroByteFilesRelaxed);
			
			this.getLogger().debug(".. job parameter [ftp_file_notification_to]               : " + fileNotificationTo);
			this.getLogger().debug(".. job parameter [ftp_file_notification_cc]               : " + fileNotificationCC);
			this.getLogger().debug(".. job parameter [ftp_file_notification_bcc]              : " + fileNotificationBCC);
			this.getLogger().debug(".. job parameter [ftp_file_notification_subject]          : " + fileNotificationSubject);
			this.getLogger().debug(".. job parameter [ftp_file_notification_body]             : " + fileNotificationBody);
			
			this.getLogger().debug(".. job parameter [ftp_file_zero_byte_notification_to]     : " + fileZeroByteNotificationTo);
			this.getLogger().debug(".. job parameter [ftp_file_zero_byte_notification_cc]     : " + fileZeroByteNotificationCC);
			this.getLogger().debug(".. job parameter [ftp_file_zero_byte_notification_bcc]    : " + fileZeroByteNotificationBCC);
			this.getLogger().debug(".. job parameter [ftp_file_zero_byte_notification_subject]: " + fileZeroByteNotificationSubject);
			this.getLogger().debug(".. job parameter [ftp_file_zero_byte_notification_body]   : " + fileZeroByteNotificationBody);
			
			this.getLogger().debug(".. job parameter [replacing]                              : " + replacing);
			this.getLogger().debug(".. job parameter [replacement]                            : " + replacement);
			
			this.getLogger().debug(".. job parameter [recursive]                              : " + recursive);
			
			this.getLogger().debug(".. job parameter [ftp_poll_timeout]                       : " + pollTimeout);
			this.getLogger().debug(".. job parameter [ftp_poll_interval]                      : " + pollIntervall);
			this.getLogger().debug(".. job parameter [ftp_poll_minfiles]                      : " + pollMinFiles);
			
			
			this.getLogger().debug(".. job parameter [ftp_file_path]                          : " + filePath);
			
			try { // to check parameters
				if (host == null || host.length() == 0) throw new Exception("no host was specified");
				if (user == null || user.length() == 0) throw new Exception("no user was specified");			    	
				//if (fileSpec == null || fileSpec.length() == 0) throw new Exception("no file specification was given");
				if(sosString.parseToString(filePath).length() > 0) {
				  if(pollMinFiles > 1) {
					  getLogger().warn("parameter [ftp_poll_minfiles] should not be greater than 1 because Parameter ftp_file_path has been specified. ");					  
				  }
				  if(sosString.parseToString(fileSpec).length() > 0 &&  !fileSpec.equals(".*") ) {
					  // getLogger().warn("parameter [ftp_file_spec] will be ignored because parameter ftp_file_path has been specified. ");
				  }
				  
				  if(sosString.parseToString(remoteDir).length() > 0 &&  !remoteDir.equals("./") ) {
					  // getLogger().warn("parameter [ftp_remote_dir] will be ignored because parameter ftp_file_path has been specified. ");
				  }
				  
				}
			} catch (Exception e) {
				rc = false;
				throw (new Exception("invalid or insufficient parameters: " + e.getMessage()));
			}
			
			
			try { // to process ftp
                this.getLogger().info("connecting to host " + host + ", port " + port + ", local directory " + localDir + ", remote directory " + remoteDir + 
                        (isFilePath ? ", file " + filePath : ", file specification " + fileSpec) );
                
				ftpClient = new SOSFTP(host, port);
				this.getLogger().debug("..ftp server reply [init] [host=" + host + "], [port="+ port + "]: " + ftpClient.getReplyString() );

				if (account != null && account.length() > 0) {
					isLoggedIn = ftpClient.login(user, password, account);
					this.getLogger().debug("..ftp server reply [login] [user=" + user + "], [" + account + "]: " + ftpClient.getReplyString() );
				} else {
					isLoggedIn = ftpClient.login(user, password);
					this.getLogger().debug("..ftp server reply [login] [user=" + user + "], [" + account + "]: " + ftpClient.getReplyString() );
				}
				
				if (!isLoggedIn || ftpClient.getReplyCode() > ERROR_CODE) {
					throw new Exception("..ftp server reply [login failed] [user=" + user + "], [" + account + "]: " + ftpClient.getReplyString() );
				}
				
				if (passiveMode) {
					ftpClient.passive();
					if (ftpClient.getReplyCode() > ERROR_CODE) {
						throw new Exception("..ftp server reply [passive]: "  + ftpClient.getReplyString());
					} else {
						this.getLogger().debug("..ftp server reply [passive]: "  + ftpClient.getReplyString());
					}
					
				}			    			
				
				if (transferMode.equalsIgnoreCase("ascii")) {
					if (ftpClient.ascii()) {
						this.getLogger().debug("..using ASCII mode for file transfer");
						this.getLogger().debug("..ftp server reply [ascii]: "  + ftpClient.getReplyString());
					} else {
						throw new Exception(".. could not switch to ASCII mode for file transfer ..ftp server reply [ascii]: "  + ftpClient.getReplyString());
					}
				} else {
					if (ftpClient.binary()) {			        	
						this.getLogger().debug("using binary mode for file transfers.");
						this.getLogger().debug("..ftp server reply [binary]: "  + ftpClient.getReplyString());
					} else {
						throw new Exception(".. could not switch to binary mode for file transfer ..ftp server reply [ascii]: "  + ftpClient.getReplyString());
					}
				}
				
				if(isFilePath) {
					String currRemoteDir = new File(filePath).getParent().replaceAll("\\\\", "/");
					if(sosString.parseToString(currRemoteDir).length() > 0) {
						ftpClient.cd(currRemoteDir);
						if (ftpClient.getReplyCode() > ERROR_CODE) {
							throw new Exception("..ftp server reply [cd] [directory ftp_file_path=" + currRemoteDir + "]: "  + ftpClient.getReplyString());
						} else {
							getLogger().debug("..ftp server reply [cd] [directory ftp_file_path=" + currRemoteDir +"]: "  + ftpClient.getReplyString() );
						}
					}
				} else if (sosString.parseToString(remoteDir).length() > 0) {
					ftpClient.cd(remoteDir);
					if (ftpClient.getReplyCode() > ERROR_CODE) {
						throw new Exception("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString());
					} else {
						getLogger().debug("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString() );
					}
				}
				
				
				Vector filelist = null;
				
				if (isFilePath) {
					this.getLogger().info("local directory: " + localDir + ", remote file: " + filePath);
					filelist = new Vector();					
					filelist.add(filePath);		
					fileSpec = ".*";
				} else {
					this.getLogger().info("local directory: " + localDir + ", remote directory: " + remoteDir + ", file specification: " + fileSpec);
					filelist = ftpClient.nList(recursive);
				}
				int count = 0;
				int zeroByteCount = 0;
				
				if (ftpClient.getReplyCode() > ERROR_CODE) {
					// no error checking for empty directories
					// throw new Exception("..error occurred looking up remote files [" + remoteDir + "]: "  + ftpClient.getReplyString());
				} else {
					getLogger().debug("..ftp server reply [nlist]: " + ftpClient.getReplyString() );
				}
				
				Pattern pattern = Pattern.compile(fileSpec, 0);
				Iterator iterator = filelist.iterator();
				
				if (pollTimeout>0){
					// before any processing, check if files are there
					boolean done= false;
					boolean giveUpPoll = false;
					double delay = pollIntervall;
					double nrOfTries = (pollTimeout*60)/delay;
					int tries=0;
					while(!done && !giveUpPoll){						
						tries++;
						int matchedFiles=0;						
						while(iterator.hasNext()) {
							String fileName = (String)iterator.next();
							if (isFilePath) {
								matchedFiles++;								
							} else {
								Matcher matcher = pattern.matcher(fileName);
								if (matcher.find()) matchedFiles++;
							}
						}
						getLogger().debug3(matchedFiles +" matching files found");
						if (matchedFiles < pollMinFiles){							
							if (spooler_job.order_queue() != null && spooler_task.order()!=null){
								getLogger().debug3(pollMinFiles +" files needed, setting back order.");
								spooler_job.set_delay_order_after_setback(1,delay);							
								spooler_job.set_max_order_setbacks((int) nrOfTries);
								spooler_task.order().setback();
								done=true;
								Variable_set orderParams = spooler_task.order().params();
								String setbackCount = orderParams.value("setback_count");
								getLogger().debug9("setback_count read: "+setbackCount);
								int iSetbackCount = 1;
								if (setbackCount!=null && setbackCount.length()>0){
									iSetbackCount = Integer.parseInt(setbackCount);
									iSetbackCount++;
								}
								getLogger().debug9("setback_count is now: "+iSetbackCount+" , maximum nr of tries: "+nrOfTries);
								orderParams.set_var("setback_count",""+iSetbackCount);
								if (iSetbackCount>nrOfTries){
									orderParams.set_var("setback_count","");
									giveUpPoll=true;
								} else return true;
							} else{// simple job
								if (tries<nrOfTries){
									Thread.sleep((long)delay * 1000);
									spooler_job.set_state_text("Polling for files... ");
									if (isFilePath) {			    	
										filelist = new Vector();
										filelist.add(filePath);			 			    	
									} else {
										filelist = ftpClient.nList(recursive);
									}
									iterator = filelist.iterator();
								} else {
									giveUpPoll = true;
								}
							}
						} else{
							done=true;
							spooler_job.set_state_text("");
						}
						if (giveUpPoll){
							String message = "Failed to find at least "+pollMinFiles+" files matching \""+ 
							                 fileSpec +"\" after triggering for "+pollTimeout+" minutes.";
							if (matchedFiles>0) message+=" (Only "+matchedFiles+" files were found)";
							if (forceFiles) {
								spooler_log.warn(message);
								String body = message +"\n";
								body += "See attached logfile for details.";
								spooler_log.mail().set_body(body);
								spooler_task.end();
							}else {
								spooler_log.info(message);
								if (spooler_job.order_queue()!=null) return true;
							}
							return false;
						}
					}					
				}
				
				iterator = filelist.iterator();
				while(iterator.hasNext()) {
					String fileName = (String)iterator.next();
					Matcher matcher = pattern.matcher(fileName);
					File transferFile = null; 
					
					if(recursive && !isFilePath) {
						if (sosString.parseToString(remoteDir).length() > 0) {
							ftpClient.cd(remoteDir);
							if (ftpClient.getReplyCode() > ERROR_CODE) {
								throw new Exception("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString());
							} else {
								getLogger().debug("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString() );
							}
						}
					}
					
					if (matcher.find() || isFilePath) {
						                    	
						if (replacement != null && replacement.length() > 0 && replacing != null && replacing.length() > 0) {
							String currTransferFilename = SOSFileOperations.getReplacementFilename(fileName, replacing, replacement);
							this.getLogger().info(" transfer file [" + fileName + "] is renamed to " + currTransferFilename);
							if(isFilePath) {
								transferFile = new File(localDir 
										+ (localDir.endsWith("/") || localDir.endsWith("\\") ? "" : "/") 
										+ new File(currTransferFilename).getName());
							} else {
								transferFile = new File(localDir 
										+ (localDir.endsWith("/") || localDir.endsWith("\\") ? "" : "/") 
										+ currTransferFilename);
							}
						} else {
							if(isFilePath) {
								transferFile = new File(localDir 
										+ (localDir.endsWith("/") || localDir.endsWith("\\") ? "" : "/") 
										+ new File(fileName).getName());
							} else {
								transferFile = new File(localDir 
										+ (localDir.endsWith("/") || localDir.endsWith("\\") ? "" : "/") 
										+ fileName);
							}
						}
						
						File transFile = transferFile;
						if(transFile.getParent() != null && !transFile.getParentFile().exists()){        		        	         		        	
							//Unterverzeichnisse mit bercksichtigen
							if(transFile.getParentFile().mkdirs()) {        		    
								this.getLogger().info("..create parent directory [" + transFile.getParentFile() + "]");
							} else {
								throw new Exception ("..error occurred creating directory [" + transFile.getParentFile() + "]");
							}
						}
						
						if (!appendFiles && !overwriteFiles && transferFile.exists()) {
							this.getLogger().info("..ftp transfer skipped for file [no overwrite]: " + transferFile.getName());
							continue;
						}
						
						long bytesSent = 0;
						
						if (!appendFiles && atomicSuffix != null && atomicSuffix.length() > 0) {
							File atomicFile = new File(transferFile.getAbsolutePath() + atomicSuffix);
							
							File file = new File(fileName); 
							if(recursive) {
								if(file.getParent() != null && !isFilePath) {
									String[] splitParent = file.getParent().split("\\\\");
									for (int i=0; i<splitParent.length; i++) {
										if(sosString.parseToString(splitParent[i]).length() > 0) {
											ftpClient.cd(splitParent[i]);
											if (ftpClient.getReplyCode() > ERROR_CODE) {
												throw new Exception("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString());
											} else {
												getLogger().debug("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString() );
											}
										}
									}
								}
							}
							
							
							bytesSent = this.transferFile(ftpClient, new File(file.getName()), atomicFile, checkRetry, checkInterval, appendFiles);
							
							if (transferFile.exists()) {
								if (overwriteFiles) {
									if (!transferFile.delete()) {
										throw new Exception("could not overwrite local file: " + transferFile.getAbsolutePath());
									}
								}
							}
							
							if (bytesSent <= 0 && !zeroByteFiles && zeroByteFilesStrict) {
								this.getLogger().info("removing local file : " + transferFile.getAbsolutePath() + " due to zero byte strict constraint");
								if (!atomicFile.delete()) {
									throw new Exception("..error occurred, could not remove temporary file: " + atomicFile.getAbsolutePath());
								}
								fileZeroByteNotificationBody += transferFile.getName() + "\n";
								zeroByteCount++;
							} else if (bytesSent <= 0 && !zeroByteFiles) {
								checkFileList.put(atomicFile, transferFile);
							} else {
								this.getLogger().info("renaming local temporary file : " + transferFile.getAbsolutePath());
								if (!atomicFile.renameTo(transferFile)) {
									throw new Exception("could not rename temporary file [" + atomicFile.getCanonicalPath() + "] to: " + transferFile.getAbsolutePath());
								}
								fileNotificationBody += transferFile.getName() + "\n";
								transferFileList.add(transferFile);
								count++;
							}
						} else {
							
							File file = new File(fileName); 
							if(recursive) {
								if(file.getParent() != null) {
									String[] splitParent = file.getParent().split("\\\\");
									for (int i=0; i<splitParent.length; i++) {
										ftpClient.cd(splitParent[i]);
										if (ftpClient.getReplyCode() > ERROR_CODE) {
											throw new Exception("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString());
										} else {
											getLogger().debug("..ftp server reply [cd] [remoteDir=" + remoteDir + "]: "  + ftpClient.getReplyString() );
										}
									}
								}
							}
							
							bytesSent = this.transferFile(ftpClient, new File(file.getName()), transferFile, checkRetry, checkInterval, appendFiles);
							
							if (bytesSent <= 0 && !zeroByteFiles && zeroByteFilesStrict) {
								this.getLogger().info("removing local file : " + transferFile.getAbsolutePath() + " due to zero byte strict constraint");
								if (!transferFile.delete()) {
									throw new Exception("..error occurred, could not remove temporary file: " + transferFile.getAbsolutePath());
								}
								fileZeroByteNotificationBody += transferFile.getName() + "\n";
								zeroByteCount++;
							} else if (bytesSent <= 0 && !zeroByteFiles) {
								checkFileList.put(transferFile, transferFile);
							} else {
								fileNotificationBody += transferFile.getName() + "\n";
								transferFileList.add(transferFile);
								count++;
							}
						}
						
						
						if (removeFiles) {
							boolean ok = ftpClient.delete(new File(fileName).getName());                        	
							if (ftpClient.getReplyCode() > ERROR_CODE) {
								throw new Exception("..error occurred removing remote file [" + transferFile.getName() + "]: "  + ftpClient.getReplyString());
							} else if (!ok) {
								throw new Exception("..error occurred, could not remove remote file [" + transferFile.getName() + "]: " + ftpClient.getReplyString());
							} else {
								this.getLogger().info("removing remote file: " + transferFile.getName());
							}
						}
					}
				}
				
				
				if (!zeroByteFiles) {
					Iterator checkFileListIterator = checkFileList.keySet().iterator();
					while(checkFileListIterator.hasNext()) {
						File checkFile = (File) checkFileListIterator.next();
						File transferFile = (File) checkFileList.get(checkFile);
						if (checkFile == null || transferFile == null) throw new Exception("..error occurred, empty file list is corrupted");
						
						// if no files with more than zero byte have been transferred then all zero byte files will be removed
						if (transferFileList.isEmpty() || zeroByteFilesRelaxed) {
							this.getLogger().info("removing local temporary file : " + checkFile.getAbsolutePath() + " due to zero byte constraint");
							if (!checkFile.delete()) {
								throw new Exception("could not remove temporary file: " + checkFile.getAbsolutePath());
							}
							zeroByteCount++;
							fileZeroByteNotificationBody += transferFile.getName() + "\n";
							// otherwise rename files with atomic suffixes to their target names
						} else {
							if (atomicSuffix != null && atomicSuffix.length() > 0 && checkFile.getAbsolutePath().endsWith(atomicSuffix)) {
								this.getLogger().info("renaming local temporary file: " + transferFile.getAbsolutePath());
								if (!checkFile.renameTo(transferFile)) {
									throw new Exception("could not rename temporary file [" + checkFile.getCanonicalPath() + "] to: " + transferFile.getAbsolutePath());
								}
							}
							count++;
							fileNotificationBody += transferFile.getName() + "\n";
						}
					}
				}
				
				
				if (zeroByteCount > 0 && fileZeroByteNotificationTo != null && fileZeroByteNotificationTo.length() > 0) {
					sendMail(fileZeroByteNotificationTo, fileZeroByteNotificationCC, fileZeroByteNotificationBCC, fileZeroByteNotificationSubject, fileZeroByteNotificationBody);
				}
				
				if (count > 0 && fileNotificationTo != null && fileNotificationTo.length() > 0) {
					sendMail(fileNotificationTo, fileNotificationCC, fileNotificationBCC, fileNotificationSubject, fileNotificationBody);
				}
				
				
				switch (count) {
				case 0: if (zeroByteCount > 0 && zeroByteFilesRelaxed) {
					        spooler_job.set_state_text("no matching files found, " + zeroByteCount + " zero byte files skipped");
					        this.getLogger().info("no matching files found, " + zeroByteCount + " zero byte files skipped");
				        } else if (zeroByteCount > 0 && zeroByteFilesStrict) {
				            throw new Exception("zero byte file(s) found");
				        } else if (forceFiles) { 
				            throw new Exception("no matching files found");
				        } else {
				            spooler_job.set_state_text("no matching files found");
				            this.getLogger().info("no matching files found");
				        }
				
                        if (!forceFiles) {
                            return false;
                        } else {
                            rc = (!forceFiles ? true : !zeroByteFilesRelaxed);
                        }
                        break;
				case 1: this.getLogger().info("1 file received" + ((zeroByteCount > 0) ? ", " + zeroByteCount + " files skipped due to zero byte constraint" : "") ); 
				        spooler_job.set_state_text("1 file received" + ((zeroByteCount > 0) ? ", " + zeroByteCount + " files skipped due to zero byte constraint" : "") );
				        rc = true; 
				        break;
				default: 	this.getLogger().info(count + " files received" + ((zeroByteCount > 0) ? ", " + zeroByteCount + " files skipped due to zero byte constraint" : "") ); 
				          spooler_job.set_state_text(count + " files received" + ((zeroByteCount > 0) ? ", " + zeroByteCount + " files skipped due to zero byte constraint" : "") );
				          rc = true; 
				          break;
				}
				
				String fileNames="";
				String filePaths="";
				Iterator transferredIterator = transferFileList.iterator();
				while (transferredIterator.hasNext()){
					File curFile = (File) transferredIterator.next();
					filePaths += curFile.getAbsolutePath();
					fileNames += curFile.getName();
					if (transferredIterator.hasNext()) {
						filePaths+=";";
						fileNames+=";";
					}
				}
				// return the number of transferred files and filenames
				if (spooler_job.order_queue() != null) {
					if (spooler_task.order() != null && spooler_task.order().params() != null) {
						spooler_task.order().params().set_var("ftp_result_files", Integer.toString(count));
						spooler_task.order().params().set_var("ftp_result_zero_byte_files", Integer.toString(zeroByteCount ));
						spooler_task.order().params().set_var("ftp_result_filenames", fileNames);
						spooler_task.order().params().set_var("ftp_result_filepaths", filePaths);
					}
				} else {
					if (spooler_task.params() != null){ 
						spooler_task.params().set_var("ftp_result_files", Integer.toString(count));
						spooler_task.params().set_var("ftp_result_zero_byte_files", Integer.toString(zeroByteCount));
						spooler_task.params().set_var("ftp_result_filenames", fileNames);
						spooler_task.params().set_var("ftp_result_filepaths", filePaths);
					}
				}
				
				return (spooler_task.job().order_queue() == null) ? false : rc;			    			    
				
			} catch (Exception e) {
				rc = false;
				spooler_job.set_state_text("could not process file transfer: " + e.getMessage());
				throw (new Exception("could not process file transfer: " + e.getMessage()));
			} finally {
				if (ftpClient != null) {
					if (isLoggedIn) try { ftpClient.logout(); } catch (Exception e) {} // no error handling
					if (ftpClient.isConnected()) try { ftpClient.disconnect(); } catch(Exception e) {} // no error handling
				}
			}
			
		} catch (Exception e){
			spooler_log.warn ("ftp processing failed: " + e.getMessage());
			return false;
		}
	}
	
	
	private long transferFile(SOSFTP ftpClient, File sourceFile, File targetFile, long checkRetry, long checkInterval, 
			boolean appendFiles) throws Exception {
		
		long retry = (checkRetry > 0) ? checkRetry : 0;
		long interval = (checkInterval > 0) ? checkInterval : 60;
		long currentBytesSent = 0;
		long previousBytesSent = 0;
		
		try {
			this.getLogger().info("receiving file: " + targetFile.getAbsolutePath() + " " + ftpClient.size(sourceFile.getName()) + " bytes");
			
			for(int i=-1; i<retry; i++) {
				currentBytesSent = ftpClient.getFile( sourceFile.getName(), targetFile.getAbsolutePath(), appendFiles );
				if (currentBytesSent < 0) currentBytesSent = 0;
				if (ftpClient.getReplyCode() > ERROR_CODE) {
					throw new Exception("..error occurred receiving file [" + targetFile.getAbsolutePath() + "]: "  + ftpClient.getReplyString());
				} else {
					this.getLogger().debug("..ftp server reply [getFile] [" + targetFile.getAbsolutePath() + ", size=" + currentBytesSent + "]: " + ftpClient.getReplyString());
				}
				
				if (appendFiles == false && retry > 0 && currentBytesSent != previousBytesSent) {
					this.getLogger().info("..retry " + (i+2) + " of " + retry + " to wait " + interval + "s for file transfer being completed, current file size: " + currentBytesSent + " bytes");
					try { 
						Thread.sleep(checkInterval*1000);
					} catch (InterruptedException e){} // the VM doesn't want us to sleep anymore, so get back to work
				} else {
					break;
				}
				
				previousBytesSent = currentBytesSent;
			}
			
			if (targetFile.length() > 0 && ftpClient.size(sourceFile.getName()) != currentBytesSent) {
				throw new Exception("..error occurred receiving file, local file size [" + targetFile.length() + "] does not match remote file size ["  + currentBytesSent + "]");
			}
			
			return currentBytesSent;
		} catch (Exception e){
			throw new Exception("file transfer failed: " + e.getMessage());
		}
		
	}
	
	
	/**
	 * send mail with Job Scheduler settings
	 * 
	 * @param recipient
	 * @param recipientCC carbon copy recipient
	 * @param recipientBCC blind carbon copy recipient
	 * @param subject
	 * @param body
	 * @throws Exception
	 */
	public void sendMail(String recipient, String recipientCC, String recipientBCC, String subject, String body) throws Exception {
		
		try {
			SOSMail sosMail = new SOSMail(spooler_log.mail().smtp());
			
			sosMail.setQueueDir(spooler_log.mail().queue_dir());
			sosMail.setFrom(spooler_log.mail().from());
			sosMail.setContentType("text/plain");
			sosMail.setEncoding("Base64");
			
			String recipients[] = recipient.split(",");
			for(int i=0; i<recipients.length; i++) {
				sosMail.addRecipient(recipients[i].trim());
			}
			
			String recipientsCC[] = recipientCC.split(",");
			for(int i=0; i<recipientsCC.length; i++) {
				sosMail.addCC(recipientsCC[i].trim());
			}
			
			String recipientsBCC[] = recipientBCC.split(",");
			for(int i=0; i<recipientsBCC.length; i++) {
				sosMail.addBCC(recipientsBCC[i].trim());
			}
			
			sosMail.setSubject(subject);
			sosMail.setBody(body);
			sosMail.setSOSLogger(this.getLogger());
			
			this.getLogger().info("sending mail: \n" + sosMail.dumpMessageAsString());
			
			if (!sosMail.send()){
				this.getLogger().warn("mail server is unavailable, mail for recipient [" + recipient + "] is queued in local directory [" + sosMail.getQueueDir() + "]:" + sosMail.getLastError());
			}
			
			sosMail.clearRecipients();
		} catch (Exception e) {
			throw new Exception("error occurred sending mai: " + e.getMessage());
		}
	}	
	
}
