package sos.stresstest;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.util.Properties;

import sos.connection.SOSConnection;
import sos.util.SOSString;

/**
 * @author Ron - Gerrit Vahle <rv@sos-berlin.com>
 * @version 1.0
 * @author Andreas Pschel  <andreas.pueschel@sos-berlin.com>
 * @version 1.1
 * @since 2004-11-13
 * 
 * StresstestImport importiert die vom Stresstest erzeugten Protokolldateien in eine Datenbank
 * StresstestImport stellt eine Verbindung zur Datenbank her, liest die in den Parametern
 * angegebene Protokolldatei ein, und fgt diese in die Tabelle STRESSTEST ein. Sollte die Tabelle 
 * STRESSTEST noch nicht existieren, erstellt StresstestImport diese.
 */
public class StresstestImport {

	private static String _configuration_file;
    private static String _log_file;
    private static boolean _clean;
    private static String _class = null;
    private static SOSConnection _connection;
	
  
    /**
	 * Konstruktor - Stellt Verbindung zur Datenbank her
	 *
	 * @param configuration_file Name einer Konfigurationsdatei mit den Parametern zum Datenbankverbindungsaufbau
	 * @param log_file Name der Protokolldatei
	 */
	public StresstestImport(String configuration_file, String log_file, boolean clean) throws Exception {
		
			_configuration_file = configuration_file;
			_log_file = log_file;
			_clean = clean;
			_connection = SOSConnection.createInstance(_configuration_file);
			_connection.connect();
	}

	/**
	 * Liefert die aktuell bestehende Datenbankverbindung
	 * @return aktuelle Datenbankverbindung
	 */
	public static SOSConnection get_connection() {
		
		return _connection;
	}
	
	/**
	 * Liest alle fr die Datenbankverbindung bentigten Parameter aus der Konfigurationsdatei aus
	 * @throws Exception
	 */
	private static void getConfigFile() throws Exception { 

		BufferedInputStream in = null;
		Properties config = new Properties();

		java.io.File config_file = new java.io.File(_configuration_file);
		if (!config_file.exists())
			throw (new Exception( "make sure that the file [" + _configuration_file +
                           "] does exist!"));

		in = new BufferedInputStream(new FileInputStream(_configuration_file));
		config.load(in);

		if ( SOSString.isEmpty(config.getProperty("class") ) )
			throw new Exception("class missing.");

		_class  = config.getProperty("class");
		if ( _class == null ) { _class = _class.trim(); }
    }

	/**
	 * Testet, ob die Tabelle fr die Eintrge vorhanden ist und erstellt diese wenn ntig 
	 * @throws Exception
	 */
	
	public static void cleanupImport()throws Exception{

		try{
			get_connection().execute("TRUNCATE TABLE STRESSTEST");
		} catch (Exception e){
			System.out.println(e.getMessage());
			try {
				get_connection().rollback();
			} catch (Exception f){
				System.out.println(e.getMessage());
			}
		}
	}
	
	/**
	 * Importiert den Inhalt einer Protokolldatei in die Datenbank
	 *
	 */
	public static void processImport() {
		
		String log = _log_file;
		String data = "";

		try {
			java.io.File file = new java.io.File(log);
			if(!file.exists()){
				throw new Exception("file not found: " + log);
			}
			FileInputStream fstream = new FileInputStream(log);
			DataInputStream in = new DataInputStream(fstream);

			while (in.available() !=0){
				data = in.readLine();
				String[] datas = data.split("\t");
				get_connection().execute("INSERT INTO STRESSTEST (\"ID\",\"THREAD_ID\",\"REPLAY\",\"FUNCTION\",\"DURATION\",\"CREATED\") VALUES (%timestamp_iso('"
						+datas[0]+"')"+","+datas[2]+","+datas[3]+",'"+datas[4]+"',"
						+datas[5]+",%timestamp_iso('"+datas[1]
						+"'))");
			}
			get_connection().commit();			
			
		} catch (Exception e) {
			System.out.println(e.getMessage());
			try {
				get_connection().rollback();
			} catch (Exception f){
				System.out.println(e.getMessage());
			}
		}
	}
	
	/**
	 * Testet, ob die Tabelle fr die Eintrge vorhanden ist und erstellt diese wenn ntig 
	 * @throws Exception
	 */
	
	public static void prepareImport() throws Exception {
		
        try{
        	get_connection().execute("SELECT COUNT(*) FROM STRESSTEST");
        }
        catch(Exception e){
    		System.out.println("Table STRESSTEST does not exist.");
    		System.out.println("creating ...");
    		getConfigFile();

    		if(_class.equals("SOSMySQLConnection")){
    			get_connection().execute("CREATE TABLE STRESSTEST ("
    				+"\"ID\"        datetime not null," 
					+"\"THREAD_ID\" int not null,"
					+"\"REPLAY\"    int default 0 not null,"   
					+"\"FUNCTION\"  varchar(254) not null,"             
					+"\"DURATION\"  int not null,"             
					+"\"CREATED\"   datetime not null,"        
					+ "PRIMARY KEY (\"ID\", \"THREAD_ID\", \"FUNCTION\", \"REPLAY\"))"
					);
    		}
    		if(_class.equals("SOSMSSQLConnection")){
        		get_connection().execute("CREATE TABLE STRESSTEST ("
        			+"\"ID\"        datetime not null," 
    				+"\"THREAD_ID\" int not null,"
    				+"\"REPLAY\"    int default 0 not null,"   
    				+"\"FUNCTION\"  varchar(254) not null,"             
    				+"\"DURATION\"  int not null,"             
    				+"\"CREATED\"   datetime not null,"        
    				+ "PRIMARY KEY (\"ID\", \"THREAD_ID\", \"FUNCTION\", \"REPLAY\"))"
					);
        	}
    		if(_class.equals("SOSOracleConnection")){
        		get_connection().execute("CREATE TABLE STRESSTEST ("
        			+"\"ID\"        date not null," 
    					+"\"THREAD_ID\" int not null,"
    					+"\"REPLAY\"    int default 0 not null,"   
    					+"\"FUNCTION\"  varchar2(254) not null,"             
    					+"\"DURATION\"  int not null,"             
    					+"\"CREATED\"   date not null,"        
    					+ "PRIMARY KEY (\"ID\", \"THREAD_ID\", \"FUNCTION\", \"REPLAY\"))"
						);
        		}
        }
	}
		

	/**
	 * main.
	 * <br/>
	 * Aufruf:
	 * 
	 * <p>
	 * <b>-config=</b>Konfigurationsdatei
	 * <br/>
	 * Ort der und Name der Konfigurationsdatei mit den Einzelheiten der Datenbankverbindung
	 * Standard: config/sos_setting.ini
	 * 
	 * <p>
	 * <b>-log=</b>Name der Log-Datei
	 * <br/>
	 * Exakter Name der vom Stresstest erzeugten Log-Datei, die in die Datenbank importiert werden soll
	 * Standard: stresstest.log
	 * <p>
	 *
	 * Beispiel Unix-Startscript:
	 * <br/>
	 * java -cp $CLASSPATH:./lib/msbase.jar:./lib/mssqlserver.jar:./lib/msutil.jar:./lib/mysql-connector-java-3.0.15-ga-bin.jar:./lib/ojdbc14.jar:./lib/sos.connection.jar:./lib/sos.util.jar:./lib/sos.stresstest.jar  sos.stresstest.StresstestImport -config=config/sos_settings.ini -log=stresstest.log
	 */
	
	public static void main(String[] args) throws Exception {
		
		Arguments arguments = new Arguments( args );

		StresstestImport s_import = new StresstestImport
        ( 
                new String(      arguments.as_string( "-config=", "config/sos_settings.ini" ) ),
                new String(    	 arguments.as_string( "-log=",    "stresstest.log")),
                arguments.as_string( "-clean=", "true").equalsIgnoreCase("true") ? true : false
		);		
		
		arguments.check_all_used();
		
		prepareImport();
		if (_clean) cleanupImport();
		processImport();
	}
}

