// $Id: xml.h 4580 2006-10-24 15:45:14Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#include "zschimmer.h"
#include "z_io.h"
#include "log.h"

namespace zschimmer {
namespace io {

using namespace ::std;

//--------------------------------------------------------------------------------------------const

const size_t Buffered_writer::default_buffer_byte_count = 50000;

//-------------------------------------------------------------------Filter_output_stream::obj_name

string Filter_output_stream::obj_name() const
{
    return "Filter_output_stream";// + _output_stream->obj_name();    
}

//------------------------------------------------------------------------------------Writer::write
    
void Writer::write( char c )
{
    write( Char_sequence( &c, 1 ) );
}

//--------------------------------ransparent_output_stream_writer::Transparent_output_stream_writer

Transparent_output_stream_writer::Transparent_output_stream_writer( Output_stream* output_stream )
: 
    _output_stream(output_stream ) 
{
}

//----------------------------------------------------------Transparent_output_stream_writer::write
    
void Transparent_output_stream_writer::write( const Char_sequence& s )
{ 
    _output_stream->write_bytes( Byte_sequence( (const Byte*)s.ptr(), s.length() ) ); 
}

//-----------------------------------------------------------------Buffered_writer::Buffered_writer

Buffered_writer::Buffered_writer( Writer* writer, int byte_count ) 
: 
    Filter_writer(writer), 
    _buffer_size(byte_count)
{
}

//----------------------------------------------------------------Buffered_writer::~Buffered_writer
    
Buffered_writer::~Buffered_writer()
{
    try
    {
        flush();
    }
    catch( exception& x ) { Z_LOG( __FUNCTION__ << "  " << x.what() << '\n' ); }
}

//---------------------------------------------------------------------------Buffered_writer::write

void Buffered_writer::write( const Char_sequence& char_sequence )
{
    if( _buffer.capacity() < _buffer_size )  _buffer.reserve( _buffer_size );

    size_t length = min( char_sequence.length(), _buffer_size - _buffer.length() );
    _buffer.append( char_sequence.ptr(), char_sequence.length() );

    if( _buffer.length() == _buffer_size )  flush();

    size_t position = length;
    while( position < char_sequence.length() )
    {
        size_t length = min( char_sequence.length() - position, _buffer_size - _buffer.length() );
        _buffer.append( char_sequence.ptr() + position, length );
        position += length;
        if( _buffer.length() == _buffer_size )  flush();
    }
}

//---------------------------------------------------------------------------Buffered_writer::flush

void Buffered_writer::flush()
{
    if( _buffer.length() > 0 )  
    {
        _writer->write( _buffer );
        _buffer.erase();
    }
}

//---------------------------------------------------------------------String_writer::String_writer

String_writer::String_writer()
{
}

//--------------------------------------------------------------------String_writer::~String_writer

String_writer::~String_writer()
{
}

//-------------------------------------------------------------------------------------------------

} //namespace io
} //namespace zschimmer
